@@ -65,7 +65,7 @@ else:
     x = unicode("111100001111")
 fp_read = io.StringIO(x)
 bv = BitVector( fp = fp_read )
-print("\nBit vector constructed directed from a file like object:")
+print("\nBit vector constructed directly from a file like object:")
 print(bv)                                    # 111100001111 
 
 # Construct a bit vector directly from a bit string:
@@ -155,7 +155,8 @@ print(bv3 >= bv1)                           # True
 
 # Write a bit vector to a file like object
 fp_write = io.StringIO()
-bv.write_bits_to_fileobject( fp_write )
+#bv.write_bits_to_fileobject( fp_write )
+bv.write_bits_to_stream_object( fp_write )
 print("\nGet bit vector written out to a file-like object:")
 print(fp_write.getvalue())                  # 1011 
 
@@ -224,10 +225,12 @@ print("\nPermuted and contracted form of the previous bit vector:")
 print(bv2)                                    # 1010
 
 print("\nExperiment with writing an internally generated bit vector out to a disk file:")
-bv1 = BitVector( bitstring = '00001010' ) 
+#bv1 = BitVector( bitstring = '00001010' ) 
+bv1 = BitVector( bitstring = '11100111' ) 
 FILEOUT = open( 'test.txt', 'wb' )
 bv1.write_to_file( FILEOUT )
 FILEOUT.close()
+
 bv2 = BitVector( filename = 'test.txt' )
 bv3 = bv2.read_bits_from_file( 32 )
 print("\nDisplay bit vectors written out to file and read back from the file and their respective lengths:")
@@ -285,10 +288,6 @@ print(bv3)   # 0100000100100000011010000111010101101110011001110111001001111001
 print("Test len() on the above bit vector:")
 print(len( bv3 ))                      # 64
 
-print("\nTest forming a [5:22] slice of the above bit vector:")
-bv4 = bv3[5:22]
-print(bv4)                             # 00100100000011010
-
 print("\nTest the iterator:")
 for bit in bv4:
     print(bit)                         # 0 0 1 0 0 1 0 0 0 0 0 0 1 1 0 1 0
@@ -320,6 +319,7 @@ bv = BitVector( intVal = 0, size = 8 )
 print(bv)                             # 00000000
 bv = BitVector( intVal = 1, size = 8 )    
 print(bv)                             # 00000001
+
 print("\nTesting slice assignment:")
 bv1 = BitVector( size = 25 )
 print("bv1= " + str(bv1))             # 0000000000000000000000000
@@ -336,6 +336,38 @@ print("bv1= " + str(bv1))             # 0101001010000000000001010
 bv3 = bv1[:]
 print("bv3= " + str(bv3))             # 0101001010000000000001010
 
+print("\nTesting slice assignment with negative limits in RHS slice:")
+bv1 = BitVector( size = 25 )
+print("bv1= " + str(bv1))             # 0000000000000000000000000
+bv2 = BitVector( bitstring = '1010111' )
+print("bv2= " + str(bv2))             # 1010001
+bv1[2:5]  = bv2[-7:-4]
+print("bv1= " + str(bv1))             # 0010100000000000000000000
+bv1[2:5]  = bv2[-3:]                  
+print("bv1= " + str(bv1))             # 0011100000000000000000000
+bv1[6:9]  = bv2[:-4]
+print("bv1= " + str(bv1))             # 0011101010000000000000000
+
+print("\nTesting slice assignment with negative limits in the LHS slice:")
+bv1 = BitVector( size = 25 )
+print("bv1= " + str(bv1))             # 0000000000000000000000000
+bv2 = BitVector( bitstring = '1111001' )
+print("bv2= " + str(bv2))             # 1111001
+bv1[-5:-2]  = bv2[0:3]
+print("bv1= " + str(bv1))             # 0000000000000000000011100
+bv1[-5:] = bv2[2:7]
+print("bv1= " + str(bv1))             # 0000000000000000000011001
+bv1[-3:]  = bv2[:-4]
+print("bv1= " + str(bv1))             # 0000000000000000000011111
+bv1[6:9]  = bv2[:3]
+print("bv1= " + str(bv1))             # 0000001110000000000011111
+bv1[:-20] = bv2[2:7]
+print("bv1= " + str(bv1))             # 1100101110000000000011111
+bv1[0:-20] = bv1[9:14]               
+print("bv1= " + str(bv1))             # 0000001110000000000011111
+#bv1[-15:-20] = bv2[2:7]              # error
+#bv1[3:-18] = bv2[2:7]                # error
+
 print("\nTesting reset function:")
 bv1.reset( 1 )             
 print("bv1= " + str(bv1))             # 1111111111111111111111111
@@ -433,9 +465,6 @@ b = BitVector( bitstring='000001111' )
 
 c = a.gf_multiply(b)
 print("Product of a=" + str(a) + " b=" + str(b) + " is " + str(c))
-                                                  # 10100110
-
-#sys.exit(0)
 
 print("\nTest division in GF(2^n):")
 mod = BitVector( bitstring='100011011' )          # AES modulus
@@ -560,3 +589,9 @@ print(bv)
 check = bv.test_for_primality()
 print("The primality test for " + str(int(bv)) + ": " + str(check))    
 
+print("\nTest generating min-canonical form of a BitVector instance:")
+for i in range(255,10000,1555):
+    bv = BitVector(intVal = i, size = 14)
+    print("\nbv:            %s" % str(bv))
+    print("min canonical: %s" % str(bv.min_canonical()))
+
